from libcpp cimport bool as cpp_bool
from .slice_ cimport Slice
from .status cimport Status

cdef extern from "rocksdb/iterator.h" namespace "rocksdb":
    cdef cppclass Iterator:
        cpp_bool Valid() nogil except+
        void SeekToFirst() nogil except+
        void SeekToLast() nogil except+
        void Seek(const Slice&) nogil except+
        void Next() nogil except+
        void Prev() nogil except+
        void SeekForPrev(const Slice&) nogil except+
        Slice key() nogil except+
        Slice value() nogil except+
        Status status() nogil except+



cdef extern from "rocksdb/universal_compaction.h" namespace "rocksdb":

    ctypedef enum CompactionStopStyle:
        kCompactionStopStyleSimilarSize
        kCompactionStopStyleTotalSize

    cdef cppclass CompactionOptionsUniversal:
        CompactionOptionsUniversal()

        unsigned int size_ratio
        unsigned int min_merge_width
        unsigned int max_merge_width
        unsigned int max_size_amplification_percent
        int compression_size_percent
        CompactionStopStyle stop_style



cdef extern from "<memory>" namespace "std":
    cdef cppclass shared_ptr[T]:
        shared_ptr() nogil except+
        shared_ptr(T*) nogil except+
        void reset() nogil except+
        void reset(T*) nogil except+
        T* get() nogil except+



from .slice_ cimport Slice
from libcpp.string cimport string
from libcpp cimport bool as cpp_bool
from .logger cimport Logger
from .std_memory cimport shared_ptr

cdef extern from "rocksdb/slice_transform.h" namespace "rocksdb":
    cdef cppclass SliceTransform:
        pass

ctypedef Slice (*transform_func)(
    void*,
    Logger*,
    string&,
    const Slice&)

ctypedef cpp_bool (*in_domain_func)(
    void*,
    Logger*,
    string&,
    const Slice&)

ctypedef cpp_bool (*in_range_func)(
    void*,
    Logger*,
    string&,
    const Slice&)

cdef extern from "cpp/slice_transform_wrapper.hpp" namespace "py_rocks":
    cdef cppclass SliceTransformWrapper:
        SliceTransformWrapper(
                string name,
                void*,
                transform_func,
                in_domain_func,
                in_range_func) nogil except+
        void set_info_log(shared_ptr[Logger]) nogil except+



from libc.stdint cimport uint32_t, uint8_t
from .std_memory cimport shared_ptr

cdef extern from "rocksdb/statistics.h" namespace "rocksdb":
    ctypedef enum StatsLevel:
        kExceptHistogramOrTimers
        kExceptTimers
        kExceptDetailedTimers
        kExceptTimeForMutex
        kAll

    cdef cppclass Statistics:
        void set_stats_level(StatsLevel) nogil except+



from libc.stdint cimport uint32_t
from libcpp cimport bool as cpp_bool
from .std_memory cimport shared_ptr

from .cache cimport Cache
from .filter_policy cimport FilterPolicy

cdef extern from "rocksdb/table.h" namespace "rocksdb":
    cdef cppclass TableFactory:
        TableFactory()

    ctypedef enum BlockBasedTableIndexType:
        kBinarySearch "rocksdb::BlockBasedTableOptions::IndexType::kBinarySearch"
        kHashSearch "rocksdb::BlockBasedTableOptions::IndexType::kHashSearch"

    ctypedef enum ChecksumType:
        kCRC32c
        kxxHash

    cdef cppclass BlockBasedTableOptions:
        BlockBasedTableOptions()
        BlockBasedTableIndexType index_type
        cpp_bool hash_index_allow_collision
        ChecksumType checksum
        cpp_bool no_block_cache
        size_t block_size
        int block_size_deviation
        int block_restart_interval
        cpp_bool whole_key_filtering
        shared_ptr[Cache] block_cache
        shared_ptr[Cache] block_cache_compressed
        shared_ptr[FilterPolicy] filter_policy
        cpp_bool enable_index_compression
        cpp_bool cache_index_and_filter_blocks
        int format_version

    cdef TableFactory* NewBlockBasedTableFactory(const BlockBasedTableOptions&)

    ctypedef enum EncodingType:
        kPlain
        kPrefix

    cdef cppclass PlainTableOptions:
        uint32_t user_key_len
        int bloom_bits_per_key
        double hash_table_ratio
        size_t index_sparseness
        size_t huge_page_tlb_size
        EncodingType encoding_type
        cpp_bool full_scan_mode
        cpp_bool store_index_in_file

    cdef TableFactory* NewPlainTableFactory(const PlainTableOptions&)



from libcpp.string cimport string
from libcpp cimport bool as cpp_bool
from libcpp.deque cimport deque
from .slice_ cimport Slice
from .logger cimport Logger
from .std_memory cimport shared_ptr

cdef extern from "rocksdb/merge_operator.h" namespace "rocksdb":
    cdef cppclass MergeOperator:
        pass

ctypedef cpp_bool (*merge_func)(
    void*,
    const Slice&,
    const Slice*,
    const Slice&,
    string*,
    Logger*)

ctypedef cpp_bool (*full_merge_func)(
    void* ctx,
    const Slice& key,
    const Slice* existing_value,
    const deque[string]& operand_list,
    string* new_value,
    Logger* logger)

ctypedef cpp_bool (*partial_merge_func)(
    void* ctx,
    const Slice& key,
    const Slice& left_op,
    const Slice& right_op,
    string* new_value,
    Logger* logger)

cdef extern from "cpp/merge_operator_wrapper.hpp" namespace "py_rocks":
    cdef cppclass AssociativeMergeOperatorWrapper:
        AssociativeMergeOperatorWrapper(string, void*, merge_func) nogil except+

    cdef cppclass MergeOperatorWrapper:
        MergeOperatorWrapper(
            string,
            void*,
            void*,
            full_merge_func,
            partial_merge_func) nogil except+



from libc.stdint cimport int32_t

cdef extern from "rocksdb/memtablerep.h" namespace "rocksdb":
    cdef cppclass MemTableRepFactory:
        MemTableRepFactory()

    cdef MemTableRepFactory* NewHashSkipListRepFactory(size_t, int32_t, int32_t)
    cdef MemTableRepFactory* NewHashLinkListRepFactory(size_t)

cdef extern from "cpp/memtable_factories.hpp" namespace "py_rocks":
    cdef MemTableRepFactory* NewVectorRepFactory(size_t)
    cdef MemTableRepFactory* NewSkipListFactory()



from libcpp cimport bool as cpp_bool
from libcpp.string cimport string
from libcpp.vector cimport vector
from libc.stdint cimport uint32_t
from libc.stdint cimport int64_t
from libc.stdint cimport uint64_t

from .status cimport Status
from .db cimport DB
from .env cimport Env

cdef extern from "rocksdb/utilities/backupable_db.h" namespace "rocksdb":
    ctypedef uint32_t BackupID

    cdef cppclass BackupableDBOptions:
        BackupableDBOptions(const string& backup_dir)

    cdef struct BackupInfo:
        BackupID backup_id
        int64_t timestamp
        uint64_t size

    cdef cppclass BackupEngine:
        Status CreateNewBackup(DB*, cpp_bool) nogil except+
        Status PurgeOldBackups(uint32_t) nogil except+
        Status DeleteBackup(BackupID) nogil except+
        void StopBackup() nogil except+
        void GetBackupInfo(vector[BackupInfo]*) nogil except+
        Status RestoreDBFromBackup(BackupID, string&, string&) nogil except+
        Status RestoreDBFromLatestBackup(string&, string&) nogil except+

    cdef Status BackupEngine_Open "rocksdb::BackupEngine::Open"(
            Env*,
            BackupableDBOptions&,
            BackupEngine**)



from libcpp cimport bool as cpp_bool
from libcpp.string cimport string
from libc.string cimport const_char
from .slice_ cimport Slice
from .std_memory cimport shared_ptr
from .logger cimport Logger

cdef extern from "rocksdb/filter_policy.h" namespace "rocksdb":
    cdef cppclass FilterPolicy:
        void CreateFilter(const Slice*, int, string*) nogil except+
        cpp_bool KeyMayMatch(const Slice&, const Slice&) nogil except+
        const_char* Name() nogil except+

    cdef extern const FilterPolicy* NewBloomFilterPolicy(int) nogil except+

ctypedef void (*create_filter_func)(
    void*,
    Logger*,
    string&,
    const Slice*,
    int,
    string*)

ctypedef cpp_bool (*key_may_match_func)(
    void*,
    Logger*,
    string&,
    const Slice&,
    const Slice&)

cdef extern from "cpp/filter_policy_wrapper.hpp" namespace "py_rocks":
    cdef cppclass FilterPolicyWrapper:
        FilterPolicyWrapper(
            string,
            void*,
            create_filter_func,
            key_may_match_func) nogil except+

        void set_info_log(shared_ptr[Logger]) nogil except+



from libcpp.string cimport string
from .slice_ cimport Slice
from .logger cimport Logger
from .std_memory cimport shared_ptr

cdef extern from "rocksdb/comparator.h" namespace "rocksdb":
    cdef cppclass Comparator:
        const char* Name()
        int Compare(const Slice&, const Slice&) const

    cdef extern const Comparator* BytewiseComparator() nogil except +

ctypedef int (*compare_func)(
    void*,
    Logger*,
    string&,
    const Slice&,
    const Slice&)

cdef extern from "cpp/comparator_wrapper.hpp" namespace "py_rocks":
    cdef cppclass ComparatorWrapper:
        ComparatorWrapper(string, void*, compare_func) nogil except +
        void set_info_log(shared_ptr[Logger]) nogil except+



from libcpp.string cimport string
from libcpp cimport bool as cpp_bool

cdef extern from "rocksdb/slice.h" namespace "rocksdb":
    cdef cppclass Slice:
        Slice() nogil
        Slice(const char*, size_t) nogil
        Slice(const string&) nogil
        Slice(const char*) nogil

        const char* data() nogil
        size_t size() nogil
        cpp_bool empty() nogil
        char operator[](int) nogil
        void clear() nogil
        void remove_prefix(size_t) nogil
        string ToString() nogil
        string ToString(cpp_bool) nogil
        int compare(const Slice&) nogil
        cpp_bool starts_with(const Slice&) nogil



#!/usr/bin/env python3

import platform
import setuptools
import sys


extra_compile_args = [
    '-std=c++11',
    '-O3',
    '-Wall',
    '-Wextra',
    '-Wconversion',
    '-fno-strict-aliasing',
    '-fno-rtti',
]

if platform.system() == 'Darwin':
    extra_compile_args += ['-mmacosx-version-min=10.7', '-stdlib=libc++']

if sys.version_info < (3 , 0):
    raise Exception("python-rocksdb requires Python 3.x")

setuptools.setup(
    ext_modules=[
        setuptools.Extension(
            'rocksdb._rocksdb',
            [
                'rocksdb/_rocksdb.pyx',
            ],
            extra_compile_args=extra_compile_args,
            language='c++',
            libraries=[
                'rocksdb',
                'snappy',
                'bz2',
                'z',
                'lz4',
            ],
        ),
    ],
)



from libcpp cimport bool as cpp_bool
from libcpp.string cimport string
from libcpp.vector cimport vector
from libc.stdint cimport uint64_t
from libc.stdint cimport uint32_t
from .std_memory cimport shared_ptr
from .comparator cimport Comparator
from .merge_operator cimport MergeOperator
from .logger cimport Logger
from .slice_ cimport Slice
from .snapshot cimport Snapshot
from .slice_transform cimport SliceTransform
from .table_factory cimport TableFactory
#from .statistics cimport Statistics
from .memtablerep cimport MemTableRepFactory
from .universal_compaction cimport CompactionOptionsUniversal
from .cache cimport Cache

cdef extern from "rocksdb/options.h" namespace "rocksdb":
    cdef cppclass CompressionOptions:
        int window_bits;
        int level;
        int strategy;
        uint32_t max_dict_bytes
        CompressionOptions() except +
        CompressionOptions(int, int, int, int) except +

    ctypedef enum CompactionStyle:
        kCompactionStyleLevel
        kCompactionStyleUniversal
        kCompactionStyleFIFO
        kCompactionStyleNone

    ctypedef enum CompressionType:
        kNoCompression
        kSnappyCompression
        kZlibCompression
        kBZip2Compression
        kLZ4Compression
        kLZ4HCCompression
        kXpressCompression
        kZSTD
        kZSTDNotFinalCompression
        kDisableCompressionOption

    ctypedef enum ReadTier:
        kReadAllTier
        kBlockCacheTier

    ctypedef enum CompactionPri:
        kByCompensatedSize
        kOldestLargestSeqFirst
        kOldestSmallestSeqFirst
        kMinOverlappingRatio

    # This needs to be in _rocksdb.pxd so it will export into python
    #cpdef enum AccessHint "rocksdb::DBOptions::AccessHint":
    #    NONE,
    #    NORMAL,
    #    SEQUENTIAL,
    #    WILLNEED

    cdef cppclass DBOptions:
        cpp_bool create_if_missing
        cpp_bool create_missing_column_families
        cpp_bool error_if_exists
        cpp_bool paranoid_checks
        # TODO: env
        shared_ptr[Logger] info_log
        int max_open_files
        int max_file_opening_threads
        #shared_ptr[Statistics] statistics
        cpp_bool use_fsync
        string db_log_dir
        string wal_dir
        uint64_t delete_obsolete_files_period_micros
        int max_background_jobs
        int max_background_compactions
        uint32_t max_subcompactions
        int max_background_flushes
        size_t max_log_file_size
        size_t log_file_time_to_roll
        size_t keep_log_file_num
        size_t recycle_log_file_num
        size_t stats_history_buffer_size
        uint64_t max_manifest_file_size
        int table_cache_numshardbits
        uint64_t WAL_ttl_seconds
        uint64_t WAL_size_limit_MB
        size_t manifest_preallocation_size
        cpp_bool allow_mmap_reads
        cpp_bool allow_mmap_writes
        cpp_bool use_direct_reads
        cpp_bool use_direct_io_for_flush_and_compaction
        cpp_bool allow_fallocate
        cpp_bool is_fd_close_on_exec
        cpp_bool skip_log_error_on_recovery
        unsigned int stats_dump_period_sec
        cpp_bool advise_random_on_open
        size_t db_write_buffer_size
        # AccessHint access_hint_on_compaction_start
        cpp_bool use_adaptive_mutex
        uint64_t bytes_per_sync
        cpp_bool allow_concurrent_memtable_write
        cpp_bool enable_write_thread_adaptive_yield
        shared_ptr[Cache] row_cache
        void IncreaseParallelism(int) nogil except+

    cdef cppclass ColumnFamilyOptions:
        ColumnFamilyOptions()
        ColumnFamilyOptions(const Options& options)
        const Comparator* comparator
        shared_ptr[MergeOperator] merge_operator
        # TODO: compaction_filter
        # TODO: compaction_filter_factory
        size_t write_buffer_size
        int max_write_buffer_number
        int min_write_buffer_number_to_merge
        CompressionType compression
        CompactionPri compaction_pri
        # TODO: compression_per_level
        shared_ptr[SliceTransform] prefix_extractor
        int num_levels
        int level0_file_num_compaction_trigger
        int level0_slowdown_writes_trigger
        int level0_stop_writes_trigger
        int max_mem_compaction_level
        uint64_t target_file_size_base
        int target_file_size_multiplier
        uint64_t max_bytes_for_level_base
        double max_bytes_for_level_multiplier
        vector[int] max_bytes_for_level_multiplier_additional
        int expanded_compaction_factor
        int source_compaction_factor
        int max_grandparent_overlap_factor
        cpp_bool disableDataSync
        double soft_rate_limit
        double hard_rate_limit
        unsigned int rate_limit_delay_max_milliseconds
        size_t arena_block_size
        # TODO: PrepareForBulkLoad()
        cpp_bool disable_auto_compactions
        cpp_bool purge_redundant_kvs_while_flush
        cpp_bool allow_os_buffer
        cpp_bool verify_checksums_in_compaction
        CompactionStyle compaction_style
        CompactionOptionsUniversal compaction_options_universal
        cpp_bool filter_deletes
        uint64_t max_sequential_skip_in_iterations
        shared_ptr[MemTableRepFactory] memtable_factory
        shared_ptr[TableFactory] table_factory
        # TODO: table_properties_collectors
        cpp_bool inplace_update_support
        size_t inplace_update_num_locks
        # TODO: remove options source_compaction_factor, max_grandparent_overlap_bytes and expanded_compaction_factor from document
        uint64_t max_compaction_bytes
        CompressionOptions compression_opts
        cpp_bool optimize_filters_for_hits
        cpp_bool paranoid_file_checks

    cdef cppclass Options(DBOptions, ColumnFamilyOptions):
        pass

    cdef cppclass WriteOptions:
        cpp_bool sync
        cpp_bool disableWAL

    cdef cppclass ReadOptions:
        cpp_bool verify_checksums
        cpp_bool fill_cache
        const Snapshot* snapshot
        ReadTier read_tier

    cdef cppclass FlushOptions:
        cpp_bool wait

    ctypedef enum BottommostLevelCompaction:
        blc_skip "rocksdb::BottommostLevelCompaction::kSkip"
        blc_is_filter "rocksdb::BottommostLevelCompaction::kIfHaveCompactionFilter"
        blc_force "rocksdb::BottommostLevelCompaction::kForce"

    cdef cppclass CompactRangeOptions:
        cpp_bool change_level
        int target_level
        uint32_t target_path_id
        BottommostLevelCompaction bottommost_level_compaction



from .std_memory cimport shared_ptr

cdef extern from "rocksdb/cache.h" namespace "rocksdb":
    cdef cppclass Cache:
        pass

    cdef extern shared_ptr[Cache] NewLRUCache(size_t)
    cdef extern shared_ptr[Cache] NewLRUCache(size_t, int)



from . cimport options
from libc.stdint cimport uint64_t, uint32_t
from .status cimport Status
from libcpp cimport bool as cpp_bool
from libcpp.string cimport string
from libcpp.vector cimport vector
from .slice_ cimport Slice
from .snapshot cimport Snapshot
from .iterator cimport Iterator

cdef extern from "rocksdb/write_batch.h" namespace "rocksdb":
    cdef cppclass WriteBatch:
        WriteBatch() nogil except+
        WriteBatch(string) nogil except+
        void Put(const Slice&, const Slice&) nogil except+
        void Put(ColumnFamilyHandle*, const Slice&, const Slice&) nogil except+
        void Merge(const Slice&, const Slice&) nogil except+
        void Merge(ColumnFamilyHandle*, const Slice&, const Slice&) nogil except+
        void Delete(const Slice&) nogil except+
        void Delete(ColumnFamilyHandle*, const Slice&) nogil except+
        void PutLogData(const Slice&) nogil except+
        void Clear() nogil except+
        const string& Data() nogil except+
        int Count() nogil except+

cdef extern from "cpp/write_batch_iter_helper.hpp" namespace "py_rocks":
    cdef enum BatchItemOp "RecordItemsHandler::Optype":
        BatchItemOpPut "py_rocks::RecordItemsHandler::Optype::PutRecord"
        BatchItemOpMerge "py_rocks::RecordItemsHandler::Optype::MergeRecord"
        BatchItemOpDelte "py_rocks::RecordItemsHandler::Optype::DeleteRecord"

    cdef cppclass BatchItem "py_rocks::RecordItemsHandler::BatchItem":
        BatchItemOp op
        uint32_t column_family_id
        Slice key
        Slice value

    Status get_batch_items(WriteBatch* batch, vector[BatchItem]* items)


cdef extern from "rocksdb/db.h" namespace "rocksdb":
    ctypedef uint64_t SequenceNumber
    string kDefaultColumnFamilyName

    cdef struct LiveFileMetaData:
        string name
        int level
        uint64_t size
        string smallestkey
        string largestkey
        SequenceNumber smallest_seqno
        SequenceNumber largest_seqno

    # cdef struct SstFileMetaData:
    #     uint64_t size
    #     string name
    #     uint64_t file_number
    #     string db_path
    #     string smallestkey
    #     string largestkey
    #     SequenceNumber smallest_seqno
    #     SequenceNumber largest_seqno

    # cdef struct LevelMetaData:
    #     int level
    #     uint64_t size
    #     string largestkey
    #     LiveFileMetaData files

    cdef struct ColumnFamilyMetaData:
        uint64_t size
        uint64_t file_count
        # string largestkey
        # LevelMetaData levels

    cdef cppclass Range:
        Range(const Slice&, const Slice&)

    cdef cppclass DB:
        Status Put(
            const options.WriteOptions&,
            ColumnFamilyHandle*,
            const Slice&,
            const Slice&) nogil except+

        Status Delete(
            const options.WriteOptions&,
            ColumnFamilyHandle*,
            const Slice&) nogil except+

        Status Merge(
            const options.WriteOptions&,
            ColumnFamilyHandle*,
            const Slice&,
            const Slice&) nogil except+

        Status Write(
            const options.WriteOptions&,
            WriteBatch*) nogil except+

        Status Get(
            const options.ReadOptions&,
            ColumnFamilyHandle*,
            const Slice&,
            string*) nogil except+

        vector[Status] MultiGet(
            const options.ReadOptions&,
            const vector[ColumnFamilyHandle*]&,
            const vector[Slice]&,
            vector[string]*) nogil except+

        cpp_bool KeyMayExist(
            const options.ReadOptions&,
            ColumnFamilyHandle*,
            Slice&,
            string*,
            cpp_bool*) nogil except+

        cpp_bool KeyMayExist(
            const options.ReadOptions&,
            ColumnFamilyHandle*,
            Slice&,
            string*) nogil except+

        Iterator* NewIterator(
            const options.ReadOptions&,
            ColumnFamilyHandle*) nogil except+

        void NewIterators(
            const options.ReadOptions&,
            vector[ColumnFamilyHandle*]&,
            vector[Iterator*]*) nogil except+

        const Snapshot* GetSnapshot() nogil except+

        void ReleaseSnapshot(const Snapshot*) nogil except+

        cpp_bool GetProperty(
            ColumnFamilyHandle*,
            const Slice&,
            string*) nogil except+

        void GetApproximateSizes(
            ColumnFamilyHandle*,
            const Range*
            int,
            uint64_t*) nogil except+

        Status CompactRange(
            const options.CompactRangeOptions&,
            ColumnFamilyHandle*,
            const Slice*,
            const Slice*) nogil except+

        Status CreateColumnFamily(
            const options.ColumnFamilyOptions&,
            const string&,
            ColumnFamilyHandle**) nogil except+

        Status DropColumnFamily(
            ColumnFamilyHandle*) nogil except+

        int NumberLevels(ColumnFamilyHandle*) nogil except+
        int MaxMemCompactionLevel(ColumnFamilyHandle*) nogil except+
        int Level0StopWriteTrigger(ColumnFamilyHandle*) nogil except+
        const string& GetName() nogil except+
        const options.Options& GetOptions(ColumnFamilyHandle*) nogil except+
        Status Flush(const options.FlushOptions&, ColumnFamilyHandle*) nogil except+
        Status DisableFileDeletions() nogil except+
        Status EnableFileDeletions() nogil except+
        Status Close() nogil except+

        # TODO: Status GetSortedWalFiles(VectorLogPtr& files)
        # TODO: SequenceNumber GetLatestSequenceNumber()
        # TODO: Status GetUpdatesSince(
                  # SequenceNumber seq_number,
                  # unique_ptr[TransactionLogIterator]*)

        Status DeleteFile(string) nogil except+
        void GetLiveFilesMetaData(vector[LiveFileMetaData]*) nogil except+
        void GetColumnFamilyMetaData(ColumnFamilyHandle*, ColumnFamilyMetaData*) nogil except+
        ColumnFamilyHandle* DefaultColumnFamily()


    cdef Status DB_Open "rocksdb::DB::Open"(
        const options.Options&,
        const string&,
        DB**) nogil except+

    cdef Status DB_Open_ColumnFamilies "rocksdb::DB::Open"(
        const options.Options&,
        const string&,
        const vector[ColumnFamilyDescriptor]&,
        vector[ColumnFamilyHandle*]*,
        DB**) nogil except+

    cdef Status DB_OpenForReadOnly "rocksdb::DB::OpenForReadOnly"(
        const options.Options&,
        const string&,
        DB**,
        cpp_bool) nogil except+

    cdef Status DB_OpenForReadOnly_ColumnFamilies "rocksdb::DB::OpenForReadOnly"(
        const options.Options&,
        const string&,
        const vector[ColumnFamilyDescriptor]&,
        vector[ColumnFamilyHandle*]*,
        DB**,
        cpp_bool) nogil except+

    cdef Status RepairDB(const string& dbname, const options.Options&)

    cdef Status ListColumnFamilies "rocksdb::DB::ListColumnFamilies" (
        const options.Options&,
        const string&,
        vector[string]*) nogil except+

    cdef cppclass ColumnFamilyHandle:
        const string& GetName() nogil except+
        int GetID() nogil except+

    cdef cppclass ColumnFamilyDescriptor:
        ColumnFamilyDescriptor() nogil except+
        ColumnFamilyDescriptor(
	    const string&,
            const options.ColumnFamilyOptions&) nogil except+
        string name
        options.ColumnFamilyOptions options

cdef extern from "rocksdb/convenience.h" namespace "rocksdb":
    void CancelAllBackgroundWork(DB*, cpp_bool) nogil except+



from libcpp cimport bool as cpp_bool
from libcpp.string cimport string

cdef extern from "rocksdb/status.h" namespace "rocksdb":
    cdef cppclass Status:
        Status()
        cpp_bool ok() nogil
        cpp_bool IsNotFound() nogil const
        cpp_bool IsCorruption() nogil const
        cpp_bool IsNotSupported() nogil const
        cpp_bool IsInvalidArgument() nogil const
        cpp_bool IsIOError() nogil const
        cpp_bool IsMergeInProgress() nogil const
        cpp_bool IsIncomplete() nogil const
        string ToString() nogil except+
